""" Utility functions for the class """

from datetime import datetime

mac_epoch_start = int(datetime(2001, 1, 1, 0, 0, 0).strftime('%s'))


def convert_to_database_date(date_string):
    date_ = datetime.strptime(date_string, '%Y-%m-%d %H:%M:%S')
    epoch_date = int(date_.strftime('%s'))
    diff = epoch_date - mac_epoch_start
    return diff * 1000000000


def convert_from_database_date(date_value):
    # date_ = date_value / 1000000000
    epoch_date = date_value + mac_epoch_start
    return datetime.fromtimestamp(epoch_date)




from imessagedb.chat import Chat


class Chats:
    """ All handles in the database """

    def __init__(self, database) -> None:
        """
            Parameters
            ----------
            database : imessagedb.DB
                An instance of a connected database"""

        self._database = database
        self._chat_list = {}
        self._chat_identifiers = {}
        self._chat_names = {}

        self._get_chats_from_database()
        return

    def __repr__(self) -> str:
        string_array = []
        for i in sorted(self.chat_list):
            string_array.append(str(self.chat_list[i]))
        return '\n'.join(string_array)

    def __len__(self) -> int:
        return len(self._chat_list)

    def _get_chats_from_database(self) -> None:
        self._database.connection.execute('select rowid, chat_identifier, display_name from chat')
        rows = self._database.connection.fetchall()
        for row in rows:
            rowid = row[0]
            chat_identifier = row[1]
            display_name = row[2]
            new_chat = Chat(self._database, rowid, chat_identifier, display_name)
            self.chat_list[new_chat.rowid] = new_chat

            # Add the chat to the chat_identifiers
            if new_chat.chat_identifier in self.chat_identifiers:
                self._chat_identifiers[new_chat.chat_identifier].append(new_chat)
            else:
                self._chat_identifiers[new_chat.chat_identifier] = [new_chat]

            # Add the chat to the chat_names
            if new_chat.chat_name != "":
                if new_chat.chat_name in self._chat_names:
                    self._chat_names[new_chat.chat_name].append(new_chat)
                else:
                    self._chat_names[new_chat.chat_name] = [new_chat]

        # Add the last chat date to all the chats
        for i in self.chat_list.values():
            select_string = "select " \
                    "datetime(max(message_date)/1000000000 + strftime('%s', '2001-01-01'),'unixepoch','localtime') " \
                    f"from chat_message_join cmj where chat_id = {i.rowid}"

            self._database.connection.execute(select_string)
            rows = self._database.connection.fetchall()
            i.last_message_date = rows[0][0]

        # Add the participants for all the chats
        self._database.connection.execute('select chat_id, handle_id from chat_handle_join')
        rows = self._database.connection.fetchall()
        for row in rows:
            chat_id = row[0]
            handle_id = row[1]

            self.chat_list[row[0]].add_participant(row[1])

        return

    def get_chats(self) -> str:
        """ Return a string with the list of chats in the database"""
        return_array = []
        for chat_id in sorted(self._chat_list):
            chat = self._chat_list[chat_id]
            chat_name = ""
            if chat.chat_name and chat.chat_name != '':
                chat_name = f"{chat.rowid} ({chat.chat_name}):"
            else:
                chat_name = f"{chat.rowid}:"
            chat_string = f"{chat_name} Participants: {chat.participants}, Last Message Sent: {chat.last_message_date}"
            return_array.append(chat_string)
        return '\n'.join(return_array)

    @property
    def chat_list(self) -> dict:
        """ Return the list of chats by rowid in a dict"""
        return self._chat_list

    @property
    def chat_identifiers(self) -> dict:
        """ Return the list of chats by chat identifier in a dict"""
        return self._chat_identifiers

    def __getitem__(self, item) -> Chat:
        if item in self._chat_names:
            return self._chat_names[item]

        if item in self._chat_list:
            return self._chat_list[item]

        if item in self._chat_identifiers:
            return self._chat_identifiers[item]

        raise KeyError



from imessagedb.handle import Handle


class Handles:
    """ All handles in the database """

    def __init__(self, database) -> None:
        """
            Parameters
            ----------
            database : imessagedb.DB
                An instance of a connected database
        """
        self._database = database
        self._handle_list = {}
        self._numbers = {}
        self._names = {}
        self._contact_by_name = {}
        self._contact_by_number = {}

        # Process the contacts first
        if self._database.config.has_section('CONTACTS'):
            for (key, value) in self._database.config.items('CONTACTS'):
                value = value.replace('\n', '')
                values = value.split(',')
                self._contact_by_name[key] = values
                for item in values:
                    self._contact_by_number[item] = key

        self._get_handles_from_database()
        return

    def _get_handles_from_database(self):

        self._database.connection.execute('select rowid, id, service from handle')
        rows = self._database.connection.fetchall()
        for row in rows:
            rowid = row[0]
            number = row[1]
            service = row[2]
            new_handle = Handle(self._database, rowid, number, service)
            self._handle_list[new_handle.rowid] = new_handle

            # Add the handle to the numbers dictionary
            if new_handle.number in self._numbers:
                self._numbers[new_handle.number].append(new_handle)
            else:
                self._numbers[new_handle.number] = [new_handle]

            # Add the handle to the names dictonary
            if new_handle.number in self._contact_by_number:
                name = self._contact_by_number[new_handle.number]
                if name in self._names:
                    self.names[name].append(new_handle)
                else:
                    self._names[name] = [new_handle]

    def get_handles(self) -> str:
        """ Return a string with the list of handles"""

        number_list = list(self._numbers.keys())
        return_string = '\n'.join(number_list)
        return return_string

    @property
    def handles(self) -> dict:
        """ Return the list of handles """
        return self._handle_list

    @property
    def numbers(self) -> dict:
        """ Return the list of handles indexed by the number """
        return self._numbers

    @property
    def names(self) -> dict:
        """ Return the list of handles indexed by the number """
        return self._names

    def name_for_number(self, number: str) -> str:
        if number in self._contact_by_number:
            return self._contact_by_number[number]

        return None

    def __iter__(self):
        return self._handle_list

    def __len__(self) -> int:
        return len(self._handle_list)

    def __repr__(self) -> str:
        handle_array = []
        for i in sorted(self._handle_list.keys()):
            handle_array.append(self._handle_list[i])
        return '\n'.join(map(str, handle_array))

    def __getitem__(self, item: str) -> Handle:
        if item in self._names:
            return self._names[item]

        if item in self._numbers:
            return self._numbers[item]

        raise KeyError


